@@ -25,6 +25,7 @@ SOFTWARE.
 
 /* checkpw stuff */
 
+#include <config.h>
 #include <sasl.h>
 #include <saslint.h>
 #include <saslutil.h>
@@ -45,6 +46,7 @@ SOFTWARE.
 #endif
 
 #include <sys/types.h>
+#include <sys/un.h>
 #include <ctype.h>
 
 #ifndef SASL_MINIMAL_SERVER
@@ -67,6 +69,8 @@ SOFTWARE.
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/uio.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -86,7 +90,7 @@ static int
 afs_cmu_StringToKey (str, cell, key)
 char *str;
 char *cell;                  /* cell for password */
-des_cblock key;
+des_cblock *key;
 {   char  password[8+1];              /* crypt is limited to 8 chars anyway */
     int   i;
     int   passlen;
@@ -128,10 +132,10 @@ static int
 afs_transarc_StringToKey (str, cell, key)
 char *str;
 char *cell;                  /* cell for password */
-des_cblock key;
+des_cblock *key;
 {   des_key_schedule schedule;
-    char temp_key[8];
-    char ivec[8];
+    des_cblock temp_key;
+    des_cblock ivec;
     char password[BUFSIZ];
     int  passlen;
 
@@ -140,16 +144,16 @@ des_cblock key;
         strncat (password, cell, sizeof(password)-passlen);
     if ((passlen = strlen(password)) > (int) sizeof(password)) passlen = sizeof(password);
 
-    memcpy (ivec, "kerberos", 8);
-    memcpy (temp_key, "kerberos", 8);
-    des_fixup_key_parity ((void *)temp_key);
-    des_key_sched (temp_key, schedule);
-    des_cbc_cksum (password, ivec, passlen, schedule, ivec);
+    memcpy (&ivec, "kerberos", 8);
+    memcpy (&temp_key, "kerberos", 8);
+    des_fixup_key_parity (&temp_key);
+    des_key_sched (&temp_key, schedule);
+    des_cbc_cksum ((des_cblock *)password, &ivec, passlen, schedule, &ivec);
 
-    memcpy (temp_key, ivec, 8);
-    des_fixup_key_parity ((void *)temp_key);
-    des_key_sched (temp_key, schedule);
-    des_cbc_cksum (password, (void *)key, passlen, schedule, ivec);
+    memcpy (&temp_key, &ivec, sizeof temp_key);
+    des_fixup_key_parity (&temp_key);
+    des_key_sched (&temp_key, schedule);
+    des_cbc_cksum ((des_cblock *)password, key, passlen, schedule, &ivec);
 
     des_fixup_key_parity (key);
 
@@ -158,7 +162,7 @@ des_cblock key;
 
 static int krb_afs_string_to_key(str, key, cell)
 char *str;
-des_cblock key;
+des_cblock *key;
 char *cell;                  /* cell for password */
 {
     if (strlen(str) > 8) {
@@ -187,7 +191,7 @@ char *lcase(char* str)
 static int use_key(char *user __attribute__((unused)), 
 		   char *instance __attribute__((unused)), 
 		   char *realm __attribute__((unused)), 
-		   des_cblock key, des_cblock returned_key)
+		   void *key, des_cblock *returned_key)
 {
     memcpy (returned_key, key, sizeof(des_cblock));
     return 0;
@@ -226,7 +230,8 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
     /* check to see if the user configured a srvtab */
     if (_sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context) 
 	== SASL_OK) {
-	getopt(context, NULL, "srvtab", &srvtab, NULL);
+	getopt(context, NULL, "srvtab", (const char **) &srvtab, NULL);
+
 	if (!srvtab) srvtab = "";
     }
 
@@ -237,10 +242,10 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
     krb_set_tkt_string(tfname);
 
     /* First try Kerberos string-to-key */
-    des_string_to_key(passwd, key);
+    des_string_to_key((char *) passwd, &key);
     
-    result = krb_get_in_tkt(user, "", realm,
-			    "krbtgt", realm, 1, use_key, NULL, key);
+    result = krb_get_in_tkt((char *)user, "", realm,
+			    "krbtgt", realm, 1, use_key, NULL, &key);
 
     if (result == INTK_BADPW) {
 	/* Now try andrew string-to-key */
@@ -248,8 +253,8 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
 	lcase(cell);
 	krb_afs_string_to_key(passwd, key, cell);
     
-	result = krb_get_in_tkt(user, "", realm,
-				"krbtgt", realm, 1, use_key, NULL, key);
+	result = krb_get_in_tkt((char *)user, "", realm,
+				"krbtgt", realm, 1, use_key, NULL, &key);
     }
 
     memset(key, 0, sizeof(key));
@@ -263,7 +268,9 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
     /* Check validity of returned ticket */
     gethostname(hostname, sizeof(hostname));
     strcpy(phost, krb_get_phost(hostname));
-    result = krb_mk_req(&authent, service, phost, realm, 0);
+
+    result = krb_mk_req(&authent, (char *) service, phost, realm, 0);
+
     if (result != 0) {
 	memset(&authent, 0, sizeof(authent));
 	dest_tkt();
@@ -271,7 +278,9 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
 	return SASL_FAIL;
     }
     strcpy(instance, "*");
-    result = krb_rd_req(&authent, service, instance, 0L, &kdata, srvtab); 
+
+    result = krb_rd_req(&authent, (char *) service, instance, 0L, &kdata, srvtab); 
+
     memset(&authent, 0, sizeof(authent));
     memset(kdata.session, 0, sizeof(kdata.session));
     if (result != 0 || strcmp(kdata.pname, user) != 0 || kdata.pinst[0] ||
@@ -292,8 +301,11 @@ int _sasl_kerberos_verify_password(sasl_conn_t *conn,
 
 #endif /* HAVE_KRB */
 
+#ifndef SASL_MINIMAL_SERVER
+
 int _sasl_shadow_verify_password(sasl_conn_t *conn __attribute__((unused)),
-				 const char *userid, const char *password,
+				 const char *userid,
+				 const char *password,
 				 const char **reply __attribute__((unused)) )
 {
 #ifdef HAVE_GETSPNAM
@@ -322,7 +334,7 @@ int _sasl_shadow_verify_password(sasl_conn_t *conn __attribute__((unused)),
 #endif
 
 }
-#ifndef SASL_MINIMAL_SERVER
+
 int _sasl_passwd_verify_password(sasl_conn_t *conn __attribute__((unused)),
 				 const char *userid,
 				 const char *password,
@@ -357,7 +369,7 @@ struct sasl_pam_data {
     int pam_error;
 };
 
-static int sasl_pam_conv(int num_msg, struct pam_message **msg,
+static int sasl_pam_conv(int num_msg, const struct pam_message **msg,
 			 struct pam_response **resp, void *appdata_ptr)
 {
     struct pam_response *reply = NULL;
@@ -501,7 +513,7 @@ static int parseuser(char **user, char **realm, const char *user_realm,
 	if (!r) {
 	    *realm = sasl_ALLOC(1);
 	    if (*realm) {
-		*realm[0] = '\0';
+		(*realm)[0] = '\0';
 		ret = SASL_OK;
 	    } else {
 		ret = SASL_NOMEM;
@@ -510,19 +522,16 @@ static int parseuser(char **user, char **realm, const char *user_realm,
 		ret = _sasl_strdup(input, user, NULL);
 	    }
 	} else {
-	    int i;
-
 	    r++;
 	    ret = _sasl_strdup(r, realm, NULL);
+	    *--r = '\0';
 	    *user = sasl_ALLOC(r - input + 1);
 	    if (*user) {
-		for (i = 0; input[i] != '@'; i++) {
-		    *user[i] = input[i];
-		}
-		*user[i] = '\0';
+		strcpy(*user, input);
 	    } else {
 		ret = SASL_NOMEM;
 	    }
+	    *r = '@';
 	}
     }
 
@@ -589,7 +598,7 @@ int _sasl_sasldb_set_pass(sasl_conn_t *conn,
     char *userid, *realm;
     int ret = SASL_OK;
 
-    if (*errstr) {
+    if (errstr) {
 	*errstr = NULL;
     }
 
@@ -602,7 +611,7 @@ int _sasl_sasldb_set_pass(sasl_conn_t *conn,
 	/* set the password */
 	sasl_secret_t *sec = NULL;
 	char salt[16];
-	sasl_rand_t *rpool;
+	sasl_rand_t *rpool = NULL;
 	sasl_server_getsecret_t *getsec;
 	sasl_server_putsecret_t *putsec;
 	void *context;
@@ -613,7 +622,7 @@ int _sasl_sasldb_set_pass(sasl_conn_t *conn,
 	    ret = _sasl_getcallback(conn, SASL_CB_SERVER_GETSECRET,
 				    &getsec, &context);
 	    if (ret == SASL_OK) {
-		getsec(context, "PLAIN", userid, realm, &sec);
+		ret = getsec(context, "PLAIN", userid, realm, &sec);
 		if (ret == SASL_OK) {
 		    sasl_free_secret(&sec);
 		    ret = SASL_NOCHANGE;
@@ -722,6 +731,8 @@ static int retry_writev(int fd, struct iovec *iov, int iovcnt)
     }
 }
 
+
+
 /* pwcheck daemon-authenticated login */
 int _sasl_pwcheck_verify_password(sasl_conn_t *conn,
 				  const char *userid, const char *passwd,
@@ -733,13 +744,13 @@ int _sasl_pwcheck_verify_password(sasl_conn_t *conn,
     struct iovec iov[10];
     static char response[1024];
     int start, n;
-    char *pwpath[1024];
+    char pwpath[1024];
     sasl_getopt_t *getopt;
     void *context;
 
     if (reply) { *reply = NULL; }
 
-    strcpy(pwpath, PWCHECKDIR);
+    strcpy(pwpath, "PWCHECKDIR");
     strcat(pwpath, "/pwcheck");
 
     s = socket(AF_UNIX, SOCK_STREAM, 0);
