@@ -384,11 +384,11 @@ int sasl_setprop(sasl_conn_t *conn, int propnum, const void *value)
 
 int sasl_usererr(int saslerr)
 {
-  if (saslerr==SASL_NOUSER)
-    return SASL_BADAUTH;
+    if (saslerr == SASL_NOUSER)
+	return SASL_BADAUTH;
 
-  /* return the error given; no transform necessary */
-  return saslerr;
+    /* return the error given; no transform necessary */
+    return saslerr;
 }
 
 const char *sasl_errstring(int saslerr,
@@ -517,25 +517,28 @@ static int _sasl_syslog(void *context __attribute__((unused)),
 			int priority,
 			const char *message)
 {
-  int syslog_priority;
+    int syslog_priority;
 
-  /* set syslog priority */
-  switch(priority) {
-  case SASL_LOG_ERR:
-    syslog_priority = LOG_ERR;
-  case SASL_LOG_WARNING:
-    syslog_priority = LOG_WARNING;
-  case SASL_LOG_INFO:
-    syslog_priority = LOG_INFO;
-    break;
-  default:
-    return SASL_BADPARAM;
-  }
-
-  /* do the syslog call. do not need to call openlog */
-  syslog(syslog_priority | LOG_AUTH, message);
-  
-  return SASL_OK;
+    /* set syslog priority */
+    switch(priority) {
+    case SASL_LOG_ERR:
+	syslog_priority = LOG_ERR;
+	break;
+    case SASL_LOG_WARNING:
+	syslog_priority = LOG_WARNING;
+	break;
+    case SASL_LOG_INFO:
+	syslog_priority = LOG_INFO;
+	break;
+    default:
+	syslog_priority = LOG_DEBUG;
+	break;
+    }
+    
+    /* do the syslog call. do not need to call openlog */
+    syslog(syslog_priority | LOG_AUTH, message);
+    
+    return SASL_OK;
 }
 
 
@@ -710,11 +713,11 @@ _sasl_getcallback(sasl_conn_t * conn,
     return SASL_OK;
   case SASL_CB_SERVER_GETSECRET:
     *pproc = _sasl_server_getsecret_hook;
-    *pcontext = NULL;
+    *pcontext = conn;
     return SASL_OK;
   case SASL_CB_SERVER_PUTSECRET:
     *pproc = _sasl_server_putsecret_hook;
-    *pcontext = NULL;
+    *pcontext = conn;
     return SASL_OK;
   case SASL_CB_VERIFYFILE:
     *pproc = & _sasl_verifyfile;
@@ -749,7 +752,11 @@ static int checksize(char **out, int *alloclen, int newlen)
 /* adds a string to the buffer; reallocing if need be */
 static int add_string(char **out, int *alloclen, int *outlen, char *add)
 {
-  int addlen=strlen(add); /* only compute once */
+  int addlen;
+
+  if (add==NULL) add = "(null)";
+
+  addlen=strlen(add); /* only compute once */
   if (checksize(out, alloclen, (*outlen)+addlen)!=SASL_OK)
     return SASL_NOMEM;
 
