@@ -64,7 +64,7 @@ static int alloc_key(const char *mechanism,
 }
 
 static int
-getsecret(void *context __attribute__((unused)),
+getsecret(void *context,
 	  const char *mechanism,
 	  const char *auth_identity,
 	  const char *realm,
@@ -75,6 +75,10 @@ getsecret(void *context __attribute__((unused)),
   size_t key_len;
   GDBM_FILE db;
   datum gkey, gvalue;  
+  void *cntxt;
+  sasl_getopt_t *getopt;
+  const char *path = SASL_DB_PATH;
+  sasl_conn_t *conn = context;
 
   if (! mechanism || ! auth_identity || ! secret || ! realm || ! db_ok)
     return SASL_FAIL;
@@ -84,7 +88,15 @@ getsecret(void *context __attribute__((unused)),
   if (result != SASL_OK)
     return result;
 
-  db = gdbm_open(SASL_DB_PATH, 0, GDBM_READER, S_IRUSR | S_IWUSR, NULL);
+  if (_sasl_getcallback(conn, SASL_CB_GETOPT,
+                        &getopt, &cntxt) == SASL_OK) {
+      const char *p;
+      if (getopt(cntxt, NULL, "sasldb_path", &p, NULL) == SASL_OK 
+	  && p != NULL && *p != 0) {
+          path = p;
+      }
+  }
+  db = gdbm_open((char *)path, 0, GDBM_READER, S_IRUSR | S_IWUSR, NULL);
   if (! db) {
     result = SASL_FAIL;
     goto cleanup;
@@ -119,7 +131,7 @@ getsecret(void *context __attribute__((unused)),
 }
 
 static int
-putsecret(void *context __attribute__((unused)),
+putsecret(void *context,
 	  const char *mechanism,
 	  const char *auth_identity,
 	  const char *realm,
@@ -130,6 +142,10 @@ putsecret(void *context __attribute__((unused)),
   size_t key_len;
   GDBM_FILE db;
   datum gkey;
+  void *cntxt;
+  sasl_getopt_t *getopt;
+  const char *path = SASL_DB_PATH;
+  sasl_conn_t *conn = context;
 
   if (! mechanism || ! auth_identity || ! realm)
       return SASL_FAIL;
@@ -139,9 +155,15 @@ putsecret(void *context __attribute__((unused)),
   if (result != SASL_OK)
     return result;
 
-
-
-  db = gdbm_open(SASL_DB_PATH, 0, GDBM_WRCREAT, S_IRUSR | S_IWUSR, NULL);
+  if (_sasl_getcallback(conn, SASL_CB_GETOPT,
+                        &getopt, &cntxt) == SASL_OK) {
+      const char *p;
+      if (getopt(cntxt, NULL, "sasldb_path", &p, NULL) == SASL_OK 
+	  && p != NULL && *p != 0) {
+          path = p;
+      }
+  }
+  db = gdbm_open((char *)path, 0, GDBM_WRCREAT, S_IRUSR | S_IWUSR, NULL);
   if (! db) {
     VL(("error opening password file. Do you have write permissions?\n"));
     result = SASL_FAIL;
@@ -172,10 +194,22 @@ sasl_server_putsecret_t *_sasl_db_putsecret = &putsecret;
 
 int _sasl_server_check_db(const sasl_callback_t *verifyfile_cb)
 {
+    const char *path = SASL_DB_PATH;
+    void *cntxt;
+    sasl_getopt_t *getopt;
     int ret;
 
+    if (_sasl_getcallback(NULL, SASL_CB_GETOPT,
+			  &getopt, &cntxt) == SASL_OK) {
+	const char *p;
+	if (getopt(cntxt, NULL, "sasldb_path", &p, NULL) == SASL_OK 
+	    && p != NULL && *p != 0) {
+	    path = p;
+	}
+    }
+
     ret = ((sasl_verifyfile_t *)(verifyfile_cb->proc))(verifyfile_cb->context,
-						       SASL_DB_PATH);
+						       path);
     if (ret == SASL_OK) {
 	db_ok = 1;
     }
