@@ -1,6 +1,6 @@
 /* SASL server API implementation
  * Tim Martin
- * $Id: server.c,v 1.54 1999/10/10 17:38:45 leg Exp $
+ * $Id: server.c,v 1.61 1999/12/02 19:12:06 leg Exp $
  */
 /***********************************************************
         Copyright 1998 by Carnegie Mellon University
@@ -220,6 +220,7 @@ external_server_init(sasl_utils_t *utils,
 typedef struct mechanism
 {
   int version;
+  int condition; /* set to SASL_NOUSER if no available users */
   const sasl_server_plug_t *plug;
   struct mechanism *next;
   void *library; /* this a pointer to shared library returned by dlopen 
@@ -341,6 +342,11 @@ int sasl_setpass(sasl_conn_t *conn,
 	if (tmpresult == SASL_OK) {
 	    _sasl_log(conn, SASL_LOG_INFO, m->plug->mech_name, 0, 0,
 		      "set secret for %s", user);
+
+	    m->condition = SASL_OK; /* if we previously thought the
+				       mechanism didn't have any user secrets 
+				       we now think it does */
+
 	} else if (tmpresult == SASL_NOCHANGE) {
 	    _sasl_log(conn, SASL_LOG_INFO, m->plug->mech_name, 0, 0,
 		      "secret not changed for %s", user);
@@ -424,10 +430,9 @@ static int add_plugin(void *p, void *library) {
   result = entry_point(mechlist->utils, SASL_SERVER_PLUG_VERSION, &version,
 		       &pluglist, &plugcount);
 
-  if (result != SASL_OK)
-  {
-    VL(("entry_point error %i\n",result));
-    return result;
+  if ((result != SASL_OK) && (result != SASL_NOUSER)) {
+      VL(("entry_point error %i\n",result));
+      return result;
   }
 
   /* Make sure plugin is using the same SASL version as us */
@@ -444,6 +449,9 @@ static int add_plugin(void *p, void *library) {
 
       mech->plug=pluglist++;
       mech->version = version;
+      
+      /* wheather this mech actually has any users in it's db */
+      mech->condition = result; /* SASL_OK or SASL_NOUSER */
 
       /*
        * We want plugin library to close but we only
@@ -740,32 +748,48 @@ int sasl_server_new(const char *service,
  * that this mechanism has everything we want
  */
 static int mech_permitted(sasl_conn_t *conn,
-			  const sasl_server_plug_t *plug)
+			  mechanism_t *mech)
 {
-  /* Can this plugin meet the application's security requirements? */
-  if (! plug || ! conn)
-    return 0;
-  if (plug == &external_server_mech) {
-    /* Special case for the external mechanism */
-    if (conn->props.min_ssf > conn->external.ssf
-	|| ! conn->external.auth_id)
-      return 0;
-  } else {
-    /* Generic mechanism */
-    if (plug->max_ssf < conn->props.min_ssf)
-      return 0;
-  }
+    const sasl_server_plug_t *plug = mech->plug;
+    int myflags;
 
-  /* security properties---if there are any flags that differ and are
-     in what the connection are requesting, then fail */
+    /* Can this plugin meet the application's security requirements? */
+    if (! plug || ! conn)
+	return 0;
 
-  /* do we want to special case SASL_SEC_PASS_CREDENTIALS? nah.. */
-  if (((conn->props.security_flags ^ plug->security_flags) 
-       & conn->props.security_flags) != 0) {
-      return 0;
-  }
+    if (plug == &external_server_mech) {
+	/* Special case for the external mechanism */
+	if (conn->props.min_ssf > conn->external.ssf
+	    || ! conn->external.auth_id)
+	    return 0;
+    } else {
+	/* Generic mechanism */
+	if (plug->max_ssf < conn->props.min_ssf)
+	    return 0;
+    }
 
-  return 1;
+    /* if there are no users in the secrets database we can't use this 
+       mechanism */
+    if (mech->condition == SASL_NOUSER) return 0;
+    
+    /* security properties---if there are any flags that differ and are
+       in what the connection are requesting, then fail */
+    
+    /* special case plaintext */
+    myflags = conn->props.security_flags;
+
+    /* if there's an external layer this is no longer plaintext */
+    if ((conn->props.min_ssf <= conn->external.ssf) && 
+	(conn->external.ssf > 0)) {
+	myflags &= ~SASL_SEC_NOPLAINTEXT;
+    }
+
+    /* do we want to special case SASL_SEC_PASS_CREDENTIALS? nah.. */
+    if (((myflags ^ plug->security_flags) & myflags) != 0) {
+	return 0;
+    }
+
+    return 1;
 }
 
 /* start a mechanism exchange within a connection context
@@ -798,8 +822,7 @@ int sasl_server_start(sasl_conn_t *conn,
   m=mechlist->mech_list;
 
   /* check parameters */
-  if ((mech==NULL)    ||
-      ((clientin==NULL) && (clientinlen>0)))
+  if ((conn == NULL) || (mech==NULL) || ((clientin==NULL) && (clientinlen>0)))
     return SASL_BADPARAM;
 
   if (errstr)
@@ -818,7 +841,7 @@ int sasl_server_start(sasl_conn_t *conn,
     return SASL_NOMECH;
 
   /* Make sure that we're willing to use this mech */
-  if (! mech_permitted(conn, m->plug))
+  if (! mech_permitted(conn, m))
     return SASL_NOMECH;
 
   s_conn->mech=m;
@@ -828,6 +851,7 @@ int sasl_server_start(sasl_conn_t *conn,
   s_conn->sparams->service=conn->service;
   s_conn->sparams->user_realm=s_conn->user_realm;
   s_conn->sparams->props=conn->props;
+  s_conn->sparams->external_ssf=conn->external.ssf;
 
   result = s_conn->mech->plug->mech_new(s_conn->mech->plug->glob_context,
 					s_conn->sparams,
@@ -1010,7 +1034,7 @@ int sasl_listmech(sasl_conn_t *conn,
   /* make list */
   for (lup = 0; lup < mechlist->mech_length; lup++) {
       /* currently, we don't use the "user" parameter for anything */
-      if (mech_permitted(conn, listptr->plug)) {
+      if (mech_permitted(conn, listptr)) {
 	  if (pcount != NULL)
 	      (*pcount)++;
 
@@ -1131,7 +1155,8 @@ int sasl_checkpass(sasl_conn_t *conn,
     }
 
     result = _sasl_checkpass(conn, mech, conn->service, user, pass);
-    *errstr = NULL;
+    if ( errstr !=NULL)
+      *errstr = NULL;
 
     if (result == SASL_OK) {
 	result = _sasl_strdup(user, &(conn->oparams.authid), NULL);
