@@ -1,6 +1,6 @@
 /* CRAM-MD5 SASL plugin
  * Tim Martin 
- * $Id: cram.c,v 1.40 1999/09/20 18:35:02 leg Exp $
+ * $Id: cram.c,v 1.44 1999/12/01 20:37:56 tmartin Exp $
  */
 /***********************************************************
         Copyright 1998 by Carnegie Mellon University
@@ -54,6 +54,9 @@ static const char rcsid[] = "$Implementation: Carnegie Mellon SASL " VERSION " $
 # define L_DEFAULT_GUARD (1)
 #endif
 
+/* global: if we've already set a pass entry */
+static int mydb_initialized = 0;
+
 typedef struct context {
 
   int state;    /* holds state are in */
@@ -272,7 +275,7 @@ static char *gettime(sasl_server_params_t *sparams)
 /* convert a string of 8bit chars to it's representation in hex
  * using lowercase letters
  */
-static char *convert16(unsigned char *in,int inlen,sasl_utils_t *utils)
+static char *convert16(unsigned char *in, int inlen, sasl_utils_t *utils)
 {
   static char hex[]="0123456789abcdef";
   int lup;
@@ -398,7 +401,7 @@ static int server_continue_step (void *conn_context,
     void *getsecret_context;
 
     HMAC_MD5_CTX tmphmac;
-    char digest_str[33];
+    char *digest_str;
     UINT4 digest[4];
 
     VL(("CRAM-MD5 Step 2\n"));
@@ -478,13 +481,8 @@ static int server_continue_step (void *conn_context,
 			      text->msgidlen);
     sparams->utils->hmac_md5_final((unsigned char *)&digest, &tmphmac);
 
-    /* this converts to base 16 with lower case letters 
-       we don't need to use snprintf here */
-    sprintf(digest_str, "%08x%08x%08x%08x",
-	    ntohl(digest[0]),
-	    ntohl(digest[1]),
-	    ntohl(digest[2]),
-	    ntohl(digest[3]));
+    /* convert to base 16 with lower case letters */
+    digest_str = convert16((unsigned char *) digest, 4, sparams->utils);
 
     /* free sec */
     free_secret(sparams->utils,&sec);
@@ -492,14 +490,15 @@ static int server_continue_step (void *conn_context,
     /* if same then verified 
      *  - we know digest_str is null terminated but clientin might not be
      */
-    if (strncmp(digest_str,clientin+pos+1,strlen(digest_str))!=0)
-    {
-      sparams->utils->free(userid);
-      if (errstr) {
-	  *errstr = "incorrect digest response";
-      }
-      return SASL_BADAUTH;
+    if (strncmp(digest_str,clientin+pos+1,strlen(digest_str))!=0) {
+	sparams->utils->free(userid);
+	if (errstr) {
+	    *errstr = "incorrect digest response";
+	}
+	free(digest_str);
+	return SASL_BADAUTH;
     }
+    free(digest_str);
 
     VL(("Succeeded!\n"));
 
@@ -533,6 +532,135 @@ static int server_continue_step (void *conn_context,
   return SASL_FAIL; /* should never get here */
 }
 
+/*
+ * See if there's at least one CRAM secret in the database
+ *
+ * Note: this function is duplicated in multiple plugins. If you fix
+ * something here please update the other files
+ */
+
+static int mechanism_db_filled(char *mech_name, sasl_utils_t *utils)
+{
+  sasl_secret_t *sec=NULL;
+  int result;
+  sasl_server_getsecret_t *getsecret;
+  void *getsecret_context;
+  int version = -1;
+
+  /* get callback so we can request the secret */
+  result = utils->getcallback(utils->conn,
+				       SASL_CB_SERVER_GETSECRET,
+				       &getsecret,
+				       &getsecret_context);
+  if (result != SASL_OK) {
+    VL(("result = %i trying to get secret callback\n",result));
+    return result;
+  }
+
+  if (! getsecret) {
+    VL(("Received NULL getsecret callback\n"));
+    return SASL_FAIL;
+  }
+
+  /* Request secret */
+  result = getsecret(getsecret_context, mech_name, "", "", &sec);
+
+  /* check version */
+
+  if (sec!=NULL)
+  {
+      if (sec->len >= 4)
+      {
+	  memcpy(&version, sec->data, 4); 
+	  version = ntohl(version);
+      }
+  }
+
+  if (version != CRAM_MD5_VERSION)
+  {
+      utils->log(utils->conn,
+		 0,
+		 mech_name,
+		 SASL_FAIL,
+		 0,
+		 "CRAM-MD5 secrets database has incompatible version (%d). My version (%d)",
+		 version, CRAM_MD5_VERSION);
+
+      return SASL_FAIL;
+  }
+  
+
+  if (result == SASL_NOUSER) {
+    return SASL_NOUSER;
+  }
+
+  free_secret(utils, &sec);
+
+  mydb_initialized = 1;
+
+  return SASL_OK;
+}
+
+/*
+ * Put a DUMMY entry in the db to show that there is at least one CRAM entry in the db
+ *
+ * Note: this function is duplicated in multiple plugins. If you fix
+ * something here please update the other files
+ */
+
+static int mechanism_fill_db(char *mech_name, sasl_server_params_t *sparams)
+{
+  int result;
+  sasl_server_putsecret_t *putsecret;
+  void *putsecret_context;
+  sasl_secret_t *sec = NULL;
+  int version;
+
+  /* don't do this again if it's already set */
+  if (mydb_initialized == 1)
+  {
+      return SASL_OK;
+  }
+
+  /* get the callback for saving to the password db */
+  result = sparams->utils->getcallback(sparams->utils->conn,
+				       SASL_CB_SERVER_PUTSECRET,
+				       &putsecret,
+				       &putsecret_context);
+  if (result != SASL_OK) {
+    return result;
+  }
+
+  /* allocate a secret structure that we're going to save to disk */  
+  sec=(sasl_secret_t *) sparams->utils->malloc(sizeof(sasl_secret_t)+
+					       4);
+  if (sec == NULL) {
+    result = SASL_NOMEM;
+    return result;
+  }
+  
+  /* set the size */
+  sec->len = 4;
+
+  /* and insert the data */
+  version = htonl(CRAM_MD5_VERSION);
+  memcpy(sec->data, &version, 4);
+
+  /* do the store */
+  result = putsecret(putsecret_context,
+		     mech_name, 
+		     "",
+		     "",
+		     sec);
+
+  if (result == SASL_OK)
+  {
+      mydb_initialized = 1;
+  }
+
+  return result;
+}
+
 static int
 setpass(void *glob_context __attribute__((unused)),
 	sasl_server_params_t *sparams,
@@ -575,7 +703,7 @@ setpass(void *glob_context __attribute__((unused)),
 
 	/* do the precalculation. this is what we're going to save to disk */
 	sparams->utils->hmac_md5_precalc(md5state, /* OUT */
-					 pass,     /* IN */
+					 (const unsigned char *) pass,     /* IN */
 					 passlen); /* IN */
 	
 	/* allocate a secret structure that we're going to save to disk */  
@@ -607,6 +735,14 @@ setpass(void *glob_context __attribute__((unused)),
 		       user,
 		       realm,
 		       sec);
+
+    if (result != SASL_OK) {
+	goto cleanup;
+    }
+
+    /* put entry in db to say we have at least one user */
+    result = mechanism_fill_db("CRAM-MD5", sparams);
+
  cleanup:
     if (sec) {
 	memset(sec, 0, sizeof(sasl_secret_t) + sizeof(HMAC_MD5_STATE));
@@ -639,7 +775,7 @@ static const sasl_server_plug_t plugins[] =
   }
 };
 
-int sasl_server_plug_init(sasl_utils_t *utils __attribute__((unused)),
+int sasl_server_plug_init(sasl_utils_t *utils,
 			  int maxversion,
 			  int *out_version,
 			  const sasl_server_plug_t **pluglist,
@@ -648,11 +784,16 @@ int sasl_server_plug_init(sasl_utils_t *utils __attribute__((unused)),
   if (maxversion<CRAM_MD5_VERSION)
     return SASL_BADVERS;
 
+  /* make sure there is a cram entry */
+
   *pluglist=plugins;
 
   *plugcount=1;  
   *out_version=CRAM_MD5_VERSION;
 
+  if ( mechanism_db_filled("CRAM-MD5",utils) != SASL_OK)
+    return SASL_NOUSER;
+
   return SASL_OK;
 }
 
