@@ -34,12 +34,31 @@ SOFTWARE.
 #include <sys/stat.h>
 #include <fcntl.h>
 
+
+/* either WITH_CMU_RC4 or WITH_SSL_RC4 is defined Let's make WITH_RC4
+   our general "there is rc4 support" and use CMU/SSL for the specific
+   library routines
+*/
+
+#ifdef WITH_CMU_RC4
+#define WITH_RC4
+#endif /* WITH_CMU_RC4 */
+
+#ifdef WITH_SSL_RC4
+#define WITH_RC4
+#endif /* WITH_SSL_RC4 */
+
+
+
 #ifdef WITH_DES
 #include <des.h>
 #endif /* WITH_DES */
-#ifdef WITH_RC4
+#ifdef WITH_CMU_RC4
 #include <rc4.h>
-#endif /* WITH_RC4 */
+#endif /* WITH_CMU_RC4 */
+#ifdef WITH_SSL_RC4
+#include <openssl/rc4.h>
+#endif /* WITH_SSL_RC4 */
 
 #ifdef WIN32
 # include <winsock.h>
@@ -132,6 +151,9 @@ typedef int cipher_function_t(void *,
 typedef int cipher_init_t(void *, sasl_utils_t *,
 			  char [16], char [16]);
 
+/* global: if we've already set a pass entry */
+static int mydb_initialized = 0;
+
 /* context that stores info */
 typedef struct context {
   int state;			/* state in the authentication we are in */
@@ -188,10 +210,15 @@ typedef struct context {
   des_key_schedule keysched_dec2;   /* key schedule for 3des initialization */
 #endif
 
-#ifdef WITH_RC4
+#ifdef WITH_CMU_RC4
   rc4_context_t *rc4_enc_context;
   rc4_context_t *rc4_dec_context;
-#endif /* WITH_RC4 */
+#endif /* WITH_CMU_RC4 */
+
+#ifdef WITH_SSL_RC4
+  RC4_KEY *rc4_enc_context;
+  RC4_KEY *rc4_dec_context;  
+#endif /* WITH_SSL_RC4 */
 
 } context_t;
 
@@ -202,45 +229,11 @@ typedef struct context {
 
 static int      htoi(unsigned char *hexin, int *res);
 
-#define DIGESTMD5_VERSION (3)
+#define DIGEST_MD5_VERSION (3)
 #define KEYS_FILE NULL
 
 static unsigned char *COLON = (unsigned char *) ":";
 
-
-/* These 2 functions are used for debugging */
-#if 0
-static void tmp_log_string(char *str)
-{
-  FILE *stream;
-
-  char name[100];
-  sprintf(name,"/tmp/foo%i",getpid());
-
-  stream=fopen(name,"a+");
-
-  fwrite(str, 1, strlen(str), stream);
-
-  fclose(stream);
-}
-
-static void tmp_log(char *foo, int len)
-{
-  FILE *stream;
-  int lup;
-  char name[100];
-  sprintf(name,"/tmp/foo%i",getpid());
-
-  stream=fopen(name,"a+");
-
-  for (lup=0;lup<len;lup++)
-    fprintf(stream,"%i. [%i] [%c]\n",lup,foo[lup],foo[lup]);
-
-
-  fclose(stream);
-}
-#endif /* 0 */
-
 void
 CvtHex(
        IN HASH Bin,
@@ -1152,7 +1145,7 @@ static int init_des(context_t *text,
 
 #endif /* WITH_DES */
 
-#ifdef WITH_RC4
+#ifdef WITH_CMU_RC4
 static int
 init_rc4(void *v, 
 	 sasl_utils_t *utils __attribute__((unused)),
@@ -1215,7 +1208,75 @@ enc_rc4(context_t *text,
   return SASL_OK;
 }
 
-#endif /* WITH_RC4 */
+#endif /* WITH_CMU_RC4 */
+
+#ifdef WITH_SSL_RC4
+
+static int
+init_rc4(void *v, 
+	 sasl_utils_t *utils __attribute__((unused)),
+	 char enckey[16],
+	 char deckey[16])
+{
+  context_t *text = (context_t *) v;
+
+  /* allocate rc4 context structures */
+  text->rc4_enc_context= (RC4_KEY *) text->malloc(sizeof(RC4_KEY));
+  if (text->rc4_enc_context==NULL) return SASL_NOMEM;
+
+  text->rc4_dec_context= (RC4_KEY *) text->malloc(sizeof(RC4_KEY));
+  if (text->rc4_dec_context==NULL) return SASL_NOMEM;
+
+  /* initialize them */
+  RC4_set_key(text->rc4_enc_context,16, (unsigned char *) enckey);
+  RC4_set_key(text->rc4_dec_context,16, (unsigned char *) deckey);
+
+  return SASL_OK;
+}
+
+static int
+dec_rc4(context_t *text,
+	const char *input,
+	unsigned inputlen,
+	unsigned char digest[16],
+	char *output,
+	unsigned *outputlen)
+{
+  /* decrypt the text part */
+  RC4(text->rc4_dec_context, inputlen-10, (unsigned char *) input, output);
+
+  /* decrypt the HMAC part */
+  RC4(text->rc4_dec_context, 10, (unsigned char *) input+(inputlen-10), (char *) digest);
+
+  /* no padding so we just subtract the HMAC to get the text length */
+  *outputlen=inputlen-10;
+
+  return SASL_OK;
+}
+
+static int
+enc_rc4(context_t *text,
+	const char *input,
+	unsigned inputlen,
+	unsigned char digest[16],
+	char *output,
+	unsigned *outputlen)
+{
+  /* pad is zero */
+  *outputlen = inputlen+10;
+
+  /* encrypt the text part */
+  RC4(text->rc4_enc_context, inputlen, (unsigned char *) input, output);
+
+  /* encrypt the HMAC part */
+  RC4(text->rc4_enc_context, 10, (unsigned char *) digest, (output)+inputlen);
+
+  return SASL_OK;
+}
+
+#endif /* WITH_SSL_RC4 */
+
+
 
 static int create_layer_keys(context_t *text,sasl_utils_t *utils,HASH key, int keylen,
 			     char enckey[16], char deckey[16])
@@ -1856,25 +1917,24 @@ server_continue_step(void *conn_context,
 
 
 #ifdef WITH_RC4
-  #ifdef WITH_DES
+#ifdef WITH_DES
     char           *qop = "auth,auth-int,auth-conf";
     char           *cipheropts="3des,des,rc4,rc4-40,rc4-56";
-  #else
+#else
     char           *qop = "auth,auth-int,auth-conf";
     char           *cipheropts="rc4,rc4-40,rc4-56";
-  #endif /* WITH_DES */
+#endif /* WITH_DES */
 #else
-  #ifdef WITH_DES
+#ifdef WITH_DES
     char           *qop = "auth,auth-int,auth-conf";
     char           *cipheropts="3des,des";
-  #else
+#else
     char           *qop = "auth,auth-int";
     char           *cipheropts="";
-  #endif /* WITH_DES */
+#endif /* WITH_DES */
 #endif /* WITH_RC4 */
 
     char           *charset = "utf-8";
-    /* char *algorithm="md5-sess"; */
 
 
     /*
@@ -1940,15 +2000,26 @@ server_continue_step(void *conn_context,
      * authentication exchange.
      */
 
-    if (add_to_challenge(sparams->utils, &challenge, "charset", (unsigned char *) charset, TRUE) != SASL_OK) {
+    if (add_to_challenge(sparams->utils, &challenge, "charset", (unsigned char *) charset, FALSE) != SASL_OK) {
       VL(("add_to_challenge 4 failed\n"));
       return SASL_FAIL;
     }
+
+
     /*
-     * if (add_to_challenge(sparams->utils, &challenge,"algorithm",
-     *     algorithm, TRUE)!=SASL_OK)
-     *         return SASL_FAIL;
+     * algorithm 
+     *  This directive is required for backwards compatibility with HTTP 
+     *  Digest., which supports other algorithms. . This directive is 
+     *  required and MUST appear exactly once; if not present, or if multiple 
+     *  instances are present, the client should abort the authentication 
+     *  exchange. 
+     *
+     * algorithm         = "algorithm" "=" "md5-sess" 
      */
+   
+    if (add_to_challenge(sparams->utils, &challenge,"algorithm",
+			 (unsigned char *) "md5-sess", FALSE)!=SASL_OK)
+      return SASL_FAIL;
 
     *serverout = challenge;
     *serveroutlen = strlen(*serverout);
@@ -2380,7 +2451,7 @@ server_continue_step(void *conn_context,
      */
 
     /* add to challenge */
-    if (add_to_challenge(sparams->utils, &response_auth, "rspauth", (unsigned char *) text->response_value, TRUE) != SASL_OK) {
+    if (add_to_challenge(sparams->utils, &response_auth, "rspauth", (unsigned char *) text->response_value, FALSE) != SASL_OK) {
       VL(("add_to_challenge failed\n"));
       result = SASL_FAIL;
       goto FreeAllMem;
@@ -2461,6 +2532,135 @@ server_continue_step(void *conn_context,
   return SASL_FAIL;		/* should never get here */
 }
 
+/*
+ * See if there's at least one secret in the database
+ *
+ * Note: this function is duplicated in multiple plugins. If you fix
+ * something here please update the other files
+ */
+
+static int mechanism_db_filled(char *mech_name, sasl_utils_t *utils)
+{
+  sasl_secret_t *sec=NULL;
+  int result;
+  sasl_server_getsecret_t *getsecret;
+  void *getsecret_context;
+  int tmpversion = -1;
+
+  /* get callback so we can request the secret */
+  result = utils->getcallback(utils->conn,
+			      SASL_CB_SERVER_GETSECRET,
+			      &getsecret,
+			      &getsecret_context);
+
+  if (result != SASL_OK) {
+    VL(("result = %i trying to get secret callback\n",result));
+    return result;
+  }
+
+  if (! getsecret) {
+    VL(("Received NULL getsecret callback\n"));
+    return SASL_FAIL;
+  }
+
+  /* Request secret */
+  result = getsecret(getsecret_context, mech_name, "", "", &sec);
+
+  /* check version */
+  if (sec!=NULL)
+  {
+      if (sec->len >= 4)
+      {
+	  memcpy(&tmpversion, sec->data, 4); 
+	  tmpversion = ntohl(tmpversion);
+      }
+  }
+
+  if (tmpversion != DIGEST_MD5_VERSION)
+  {
+      utils->log(utils->conn,
+		 0,
+		 mech_name,
+		 SASL_FAIL,
+		 0,
+		 "DIGEST-MD5 secrets database has incompatible version (%d). My version (%d)",
+		 tmpversion, DIGEST_MD5_VERSION);
+
+      return SASL_FAIL;
+  }
+  
+
+  if (result == SASL_NOUSER) {
+    return SASL_NOUSER;
+  }
+
+  mydb_initialized = 1;
+
+  if (sec!=NULL)
+    utils->free(sec);
+
+  return result;
+}
+
+/*
+ * Put a DUMMY entry in the db to show that there is at least one entry in the db
+ *
+ * Note: this function is duplicated in multiple plugins. If you fix
+ * something here please update the other files
+ */
+
+static int mechanism_fill_db(char *mech_name, sasl_server_params_t *sparams)
+{
+  int result;
+  sasl_server_putsecret_t *putsecret;
+  void *putsecret_context;
+  sasl_secret_t *sec = NULL;
+
+  /* don't do this again if it's already set */
+  if (mydb_initialized == 1)
+  {
+      return SASL_OK;
+  }
+
+  /* get the callback for saving to the password db */
+  result = sparams->utils->getcallback(sparams->utils->conn,
+				       SASL_CB_SERVER_PUTSECRET,
+				       &putsecret,
+				       &putsecret_context);
+  if (result != SASL_OK) {
+    return result;
+  }
+
+  /* allocate a secret structure that we're going to save to disk */  
+  sec=(sasl_secret_t *) sparams->utils->malloc(sizeof(sasl_secret_t)+
+					       4);
+  if (sec == NULL) {
+    result = SASL_NOMEM;
+    return result;
+  }
+
+  /* set the size */
+  sec->len = 4;
+
+  /* and insert the data */
+  version = htonl(DIGEST_MD5_VERSION);
+  memcpy(sec->data, &version, 4);
+
+  /* do the store */
+  result = putsecret(putsecret_context,
+		     mech_name, 
+		     "",
+		     "",
+		     sec);
+
+  if (result == SASL_OK)
+  {
+      mydb_initialized = 1;
+  }
+
+  return result;
+}
+
 static int
 setpass(void *glob_context __attribute__((unused)),
 	sasl_server_params_t * sparams,
@@ -2530,6 +2730,13 @@ setpass(void *glob_context __attribute__((unused)),
       memset(&secbuf, 0, sizeof(secbuf));
   }
 
+  if (result != SASL_OK) {
+      return result;
+  }
+
+  /* put entry in db to say we have at least one user */
+  result = mechanism_fill_db("DIGEST-MD5", sparams);
+
   return result;
 }
 
@@ -2540,11 +2747,11 @@ const sasl_server_plug_t plugins[] =
 #ifdef WITH_RC4
     128,				/* max ssf */
 #else
-  #ifdef WITH_DES
+#ifdef WITH_DES
     112,
-  #else 
+#else 
     0,
-  #endif /* WITH_DES */
+#endif /* WITH_DES */
 #endif /* WITH_RC4 */
 
     SASL_SEC_NOPLAINTEXT | SASL_SEC_NOANONYMOUS,
@@ -2574,7 +2781,12 @@ int sasl_server_plug_init(sasl_utils_t * utils __attribute__((unused)),
   *pluglist = plugins;
 
   *plugcount = 1;
-  *out_version = DIGESTMD5_VERSION;
+  *out_version = DIGEST_MD5_VERSION;
+
+  if ( mechanism_db_filled("DIGEST-MD5",utils) != SASL_OK)
+  {
+    return SASL_NOUSER;
+  }
 
   return SASL_OK;
 }
@@ -3024,6 +3236,8 @@ c_continue_step(void *conn_context,
   }
 
   if (text->state == 2) {
+    int need = 0;
+    int musthave = 0;
     unsigned char  *digesturi = NULL;
     unsigned char  *nonce = NULL;
     unsigned char  *ncvalue = (unsigned char *) "00000001";
@@ -3049,6 +3263,7 @@ c_continue_step(void *conn_context,
     int             auth_result = SASL_OK;
     int             pass_result = SASL_OK;
     int            realm_result = SASL_OK;
+    int            algorithm_count = 0;
 
     VL(("Digest-MD5 Step 2\n"));
 
@@ -3175,11 +3390,33 @@ c_continue_step(void *conn_context,
 	    } else {
 		IsUTF8 = TRUE;
 	    }
+	} else if (strcmp(name,"algorithm")==0) {
+
+	  if (strcmp(value, "md5-sess") != 0)
+	  {
+	    VL(("'algorithm' isn't 'md5-sess'\n"));
+	    result = SASL_FAIL;
+	    goto FreeAllocatedMem;
+	  }
+
+	  algorithm_count++;
+	  if (algorithm_count > 1)
+	  {
+	    VL(("Must see 'algoirthm' only once\n"));
+	    result = SASL_FAIL;
+	    goto FreeAllocatedMem;
+	  }
 	} else {
 	    VL(("unrecognized pair: ignoring\n"));
 	}
     }
 
+    if (algorithm_count != 1)
+    {
+      VL(("Must see 'algoirthm' once. Didn't see at all\n"));
+      result = SASL_FAIL;
+      goto FreeAllocatedMem;
+    }
 
     /* make callbacks */
 
@@ -3268,22 +3505,21 @@ c_continue_step(void *conn_context,
     secprops = params->props;
     external = params->external_ssf;
     /*    VL(("external ssf=%d\n", external));*/
-    if (params->props.min_ssf > 56) {
-      VL(("Minimum ssf too strong min_ssf=%i\n", params->props.min_ssf));
-      return SASL_TOOWEAK;
+    if ((int)params->props.min_ssf > 56 + external) {
+	return SASL_TOOWEAK;
+    } else if (params->props.min_ssf > params->props.max_ssf) {
+	return SASL_BADPARAM;
     }
-    /*
-     * this isn't necessary right? if (secprops.max_ssf<0) { VL (("ssf too
-     * strong")); return SASL_FAIL; }
-     */
-
-    /*    VL(("minssf=%i maxssf=%i\n", params->props.min_ssf, params->props.max_ssf));*/
 
     params->utils->free(qop);
 
-    /* if client didn't set use strongest layer */
-    if ((params->props.max_ssf > 1) &&
+    /* need bits of layer */
+    need = params->props.max_ssf - external;
+    musthave = params->props.min_ssf - external;
+
+    if ((need > 1) &&
 	((protection & DIGEST_PRIVACY) == DIGEST_PRIVACY)) {
+	/* let's find an encryption scheme that we like */
 
       oparams->encode = &privacy_encode; 
       oparams->decode = &privacy_decode;
@@ -3293,7 +3529,7 @@ c_continue_step(void *conn_context,
       /* Client request encryption, server supports it */
       /* encryption */
 #ifdef WITH_RC4
-      if ((params->props.max_ssf>=128)  && 
+      if ((need>=128)  && (musthave <=128) && 
 	  ((ciphers & CIPHER_RC4) == CIPHER_RC4)) { /* rc4 */
 	VL(("Trying to use rc4"));
 	cipher = "rc4";
@@ -3307,7 +3543,8 @@ c_continue_step(void *conn_context,
 #endif /* WITH_RC4 */
 
 #ifdef WITH_DES
-      } else if ((params->props.max_ssf>=112) && ((ciphers & CIPHER_3DES) == CIPHER_3DES)) {
+      } else if ((need>=112) && (musthave <=112) &&
+		 ((ciphers & CIPHER_3DES) == CIPHER_3DES)) {
 	VL(("Trying to use 3des"));
 	cipher = "3des";
 	text->cipher_enc=(cipher_function_t *) &enc_3des;
@@ -3318,7 +3555,8 @@ c_continue_step(void *conn_context,
 #endif /* WITH_DES */
 
 #ifdef WITH_RC4
-      } else if ((params->props.max_ssf>=56)  && ((ciphers & CIPHER_RC456) == CIPHER_RC456)) { /* rc4-56 */
+      } else if ((need>=56) && (musthave <=56) &&
+		 ((ciphers & CIPHER_RC456) == CIPHER_RC456)) { /* rc4-56 */
  	VL(("Trying to use rc4-56"));
  	cipher = "rc4-56";
  	text->cipher_enc=(cipher_function_t *) &enc_rc4;
@@ -3329,7 +3567,8 @@ c_continue_step(void *conn_context,
 #endif /* WITH_RC4 */
 
 #ifdef WITH_DES
-      } else if ((params->props.max_ssf>=55)  && ((ciphers & CIPHER_DES) == CIPHER_DES)) { /* des */
+      } else if ((need>=55) && (musthave <=55) &&
+		 ((ciphers & CIPHER_DES) == CIPHER_DES)) { /* des */
 	VL(("Trying to use des"));
 	cipher = "des";
 	text->cipher_enc=(cipher_function_t *) &enc_des;
@@ -3340,7 +3579,8 @@ c_continue_step(void *conn_context,
 #endif /* WITH_DES */
 
 #ifdef WITH_RC4
-      } else if ((params->props.max_ssf>=40)  && ((ciphers & CIPHER_RC440) == CIPHER_RC440)) { /* rc4-40 */
+      } else if ((need>=40) && (musthave <=40) &&
+		 ((ciphers & CIPHER_RC440) == CIPHER_RC440)) { /* rc4-40 */
  	VL(("Trying to use rc4-40"));
  	cipher = "rc4-40";
  	text->cipher_enc=(cipher_function_t *) &enc_rc4;
@@ -3360,7 +3600,7 @@ c_continue_step(void *conn_context,
 
     if (qop==NULL)
     {
-      if ((params->props.min_ssf <= 1) && (params->props.max_ssf >= 1) &&
+      if ((need >= 1) && (musthave <=1) &&
 	  ((protection & DIGEST_INTEGRITY) == DIGEST_INTEGRITY)) {
 	/* integrity */
 	oparams->encode = &integrity_encode;
@@ -3368,8 +3608,7 @@ c_continue_step(void *conn_context,
 	oparams->mech_ssf = 1;
 	qop = "auth-int";
 	VL(("Using integrity layer\n"));
-	
-      } else {
+      } else if (musthave <=0) {
 	/* no layer */
 	oparams->encode = NULL;
 	oparams->decode = NULL;
@@ -3383,6 +3622,10 @@ c_continue_step(void *conn_context,
 	  result = SASL_FAIL;
 	  goto FreeAllocatedMem;
 	}
+      } else {
+	  VL(("Can't find an acceptable layer\n"));
+	  result = SASL_FAIL;
+	  goto FreeAllocatedMem;
       }
     }
 
@@ -3462,7 +3705,7 @@ c_continue_step(void *conn_context,
       result = SASL_FAIL;
       goto FreeAllocatedMem;
     }
-    if (add_to_challenge(params->utils, &client_response, "qop", (unsigned char *) qop, TRUE) != SASL_OK) {
+    if (add_to_challenge(params->utils, &client_response, "qop", (unsigned char *) qop, FALSE) != SASL_OK) {
       result = SASL_FAIL;
       goto FreeAllocatedMem;
     }
@@ -3483,7 +3726,7 @@ c_continue_step(void *conn_context,
       result = SASL_FAIL;
       goto FreeAllocatedMem;
     }
-    if (add_to_challenge(params->utils, &client_response, "response", (unsigned char *) response, TRUE) != SASL_OK) {
+    if (add_to_challenge(params->utils, &client_response, "response", (unsigned char *) response, FALSE) != SASL_OK) {
 
       result = SASL_FAIL;
       goto FreeAllocatedMem;
@@ -3652,13 +3895,12 @@ const sasl_client_plug_t client_plugins[] =
 #ifdef WITH_RC4
     128,				/* max ssf */
 #else
- #ifdef WITH_DES
+#ifdef WITH_DES
     112,
- #else
+#else
     0,
- #endif /* WITH_DES */
+#endif /* WITH_DES */
 #endif /* WITH_RC4 */
-
     SASL_SEC_NOPLAINTEXT | SASL_SEC_NOANONYMOUS,
     client_required_prompts,
     NULL,
@@ -3676,13 +3918,13 @@ int             sasl_client_plug_init(sasl_utils_t * utils __attribute__((unused
 				                      int *out_version,
 		                       const sasl_client_plug_t ** pluglist,
 				                      int *plugcount) {
-  if (maxversion < DIGESTMD5_VERSION)
+  if (maxversion < DIGEST_MD5_VERSION)
     return SASL_BADVERS;
 
   *pluglist = client_plugins;
 
   *plugcount = 1;
-  *out_version = DIGESTMD5_VERSION;
+  *out_version = DIGEST_MD5_VERSION;
 
   return SASL_OK;
 }
