@@ -1,6 +1,6 @@
 /* Kerberos4 SASL plugin
  * Tim Martin 
- * $Id: kerberos4.c,v 1.40 1999/10/01 20:26:43 leg Exp $
+ * $Id: kerberos4.c,v 1.46 1999/12/02 05:21:03 tmartin Exp $
  */
 /***********************************************************
         Copyright 1998 by Carnegie Mellon University
@@ -145,7 +145,7 @@ static int privacy_encode(void *context, const char *input, unsigned inputlen,
   
   len=krb_mk_priv((char *) input, *output+4,
 		  inputlen,  text->init_keysched, 
-		  text->session, text->ip_local,
+		  &text->session, text->ip_local,
 		  text->ip_remote);
 
   *outputlen=len+4;
@@ -232,7 +232,7 @@ static int privacy_decode(void *context,
     memset(data,0,sizeof(MSG_DAT));
     
     len= krb_rd_priv((char *) text->buffer,text->size,  text->init_keysched, 
-		     text->session, text->ip_remote, text->ip_local, data);
+		     &text->session, text->ip_remote, text->ip_local, data);
 
     /* see if the krb library gave us a failure */
     if (len != 0) {
@@ -297,10 +297,8 @@ integrity_encode(void *context,
   *output=text->malloc(inputlen+40);
   if ((*output) ==NULL) return SASL_NOMEM;
 
-  len=krb_mk_safe((char *) input, (*output)+4,
-		  inputlen, /* text->keysched, */
-		  (text->session), (text->ip_local),
-		  (text->ip_remote));
+  len=krb_mk_safe((char *) input, (*output)+4, inputlen, /* text->keysched, */
+		  &text->session, text->ip_local, text->ip_remote);
 
   /* returns -1 on error */
   if (len==-1) return SASL_FAIL;
@@ -384,8 +382,8 @@ static int integrity_decode(void *context, const char *input, unsigned inputlen,
     if (data==NULL) return SASL_NOMEM;
 
     len= krb_rd_safe((char *) text->buffer,text->size, /* text->keysched, */
-		     (text->session),
-		     (text->ip_remote), (text->ip_local), data);
+		     &text->session,
+		     text->ip_remote, text->ip_local, data);
 
 
     /* see if the krb library found a problem with what we were given */
@@ -433,21 +431,21 @@ static int integrity_decode(void *context, const char *input, unsigned inputlen,
 }
 
 static int
-new_text(sasl_client_params_t *p, context_t **text)
+new_text(sasl_utils_t *utils, context_t **text)
 {
-    context_t *ret = (context_t *) p->utils->malloc(sizeof(context_t));
+    context_t *ret = (context_t *) utils->malloc(sizeof(context_t));
 
     if (ret==NULL) return SASL_NOMEM;
 
-    ret->malloc = p->utils->malloc;
-    ret->realloc = p->utils->realloc;
-    ret->free = p->utils->free;
+    ret->malloc = utils->malloc;
+    ret->realloc = utils->realloc;
+    ret->free = utils->free;
 
     ret->buffer = NULL;
     ret->bufsize = 0;
 
-    ret->time_sec=0;
-    ret->time_5ms=0;
+    ret->time_sec = 0;
+    ret->time_5ms = 0;
 
     ret->state = 0;  
     *text = ret;
@@ -466,7 +464,7 @@ server_start(void *glob_context __attribute__((unused)),
   if (errstr)
       *errstr = NULL;
 
-  return new_text(sparams, (context_t **) conn);
+  return new_text(sparams->utils, (context_t **) conn);
 }
 
 
@@ -631,11 +629,11 @@ static int server_continue_step (void *conn_context,
     memcpy(text->pname, ad.pname, sizeof(text->pname));
     memcpy(text->pinst, ad.pinst, sizeof(text->pinst));
     memcpy(text->prealm, ad.prealm, sizeof(text->prealm));
-    des_key_sched(ad.session, text->init_keysched);
+    des_key_sched(&ad.session, text->init_keysched);
 
     /* make keyschedule for encryption and decryption */
-    des_key_sched(ad.session, text->enc_keysched);
-    des_key_sched(ad.session, text->dec_keysched);
+    des_key_sched(&ad.session, text->enc_keysched);
+    des_key_sched(&ad.session, text->dec_keysched);
     
     des_ecb_encrypt((des_cblock *)sout,
 		    (des_cblock *)sout,
@@ -748,10 +746,10 @@ static int server_continue_step (void *conn_context,
       }  
 
       if (in[8]) {
-	  oparams->user = sparams->utils->malloc(strlen(in + 8) + 1);
+	  oparams->user = sparams->utils->malloc(strlen((char *) in + 8) + 1);
 	  if (oparams->user == NULL)
 	      return SASL_NOMEM;
-	  strcpy(oparams->user, in + 8);
+	  strcpy(oparams->user, (char *) in + 8);
       } else {
 	  oparams->user = sparams->utils->malloc(len + 1);
 	  strcpy(oparams->user, oparams->authid);
@@ -809,7 +807,7 @@ int sasl_server_plug_init(sasl_utils_t *utils,
 			  const sasl_server_plug_t **pluglist,
 			  int *plugcount)
 {
-    char *ret;
+    const char *ret;
     unsigned int rl;
     
     utils->getopt(utils->getopt_context, "KERBEROS_V4", "srvtab", &ret, &rl);
@@ -842,7 +840,7 @@ static int client_start(void *glob_context __attribute__((unused)),
 {
   VL(("KERBEROS_V4 Client start\n"));
 
-  return new_text(params, (context_t **) conn);
+  return new_text(params->utils, (context_t **) conn);
 }
 
 static int client_continue_step (void *conn_context,
@@ -937,6 +935,8 @@ static int client_continue_step (void *conn_context,
   /* challenge #2 */
   if (text->state==2)
   {
+    int need = 0;
+    int musthave = 0;
     int testnum;
     int nchal;    
     unsigned char sout[1024];
@@ -1047,9 +1047,9 @@ static int client_continue_step (void *conn_context,
     memcpy(text->session, text->credentials.session, 8);
 
     /* make key schedule for encryption and decryption */
-    des_key_sched(text->session, text->init_keysched);
-    des_key_sched(text->session, text->enc_keysched);
-    des_key_sched(text->session, text->dec_keysched);
+    des_key_sched(&text->session, text->init_keysched);
+    des_key_sched(&text->session, text->enc_keysched);
+    des_key_sched(&text->session, text->dec_keysched);
 
     /* decrypt from server */
     des_ecb_encrypt((des_cblock *)in, (des_cblock *)in,
@@ -1071,19 +1071,27 @@ static int client_continue_step (void *conn_context,
      * 6-8 max buffer size
      */
 
-    if (params->props.min_ssf > KRB_DES_SECURITY_BITS)
-    {
+
+    if (params->props.min_ssf > KRB_DES_SECURITY_BITS + params->external_ssf) {
       VL (("Minimum ssf too strong min_ssf=%u\n",
 	   params->props.min_ssf));
       return SASL_TOOWEAK;
+    } else if (params->props.min_ssf > params->props.max_ssf) {
+      return SASL_BADPARAM;
     }
 
     /* create stuff to send to server */
     nchal=htonl(text->challenge);
     memcpy(sout, &nchal, 4);
 
+    /* need bits of layer */
+    need = params->props.max_ssf - params->external_ssf;
+    musthave = params->props.min_ssf - params->external_ssf;
+
+    VL(("need = %i musthave = %i \n",need, musthave));
+
     if ((in[4] & KRB_SECFLAG_ENCRYPTION)
-	&& cando_sec(&params->props, KRB_SECFLAG_ENCRYPTION)) {
+	&& (need>=56) && (musthave <= 56)) {
       /* encryption */
       oparams->encode = &privacy_encode;
       oparams->decode = &privacy_decode;
@@ -1091,15 +1099,14 @@ static int client_continue_step (void *conn_context,
       sout[4] = KRB_SECFLAG_ENCRYPTION;
       VL (("Using encryption layer\n"));
     } else if ((in[4] & KRB_SECFLAG_INTEGRITY)
-	       && cando_sec(&params->props, KRB_SECFLAG_INTEGRITY)) {
+	       && (need >= 1) && (musthave <= 1)) {
       /* integrity */
       oparams->encode=&integrity_encode;
       oparams->decode=&integrity_decode;
       oparams->mech_ssf=1;
       sout[4] = KRB_SECFLAG_INTEGRITY;
       VL (("Using integrity layer\n"));
-    } else if ((in[4] & KRB_SECFLAG_NONE)
-	       && cando_sec(&params->props, KRB_SECFLAG_NONE)) {
+    } else if ((in[4] & KRB_SECFLAG_NONE) && (musthave <= 0)) {
       /* no layer */
       oparams->encode=NULL;
       oparams->decode=NULL;
@@ -1203,8 +1210,8 @@ static const sasl_client_plug_t client_plugins[] =
     "KERBEROS_V4",
     KRB_DES_SECURITY_BITS,
     SASL_SEC_NOPLAINTEXT | SASL_SEC_NOACTIVE | SASL_SEC_NOANONYMOUS,
+    client_required_prompts,
     NULL,
-    &client_required_prompts,
     &client_start,
     &client_continue_step,
     &dispose,
