@@ -196,6 +196,7 @@ main(int argc, char *argv[])
       break;
     default:
       flag_error = 1;
+      break;
     }
 
   if (optind != argc - 1)
@@ -232,19 +233,21 @@ main(int argc, char *argv[])
     flag_pipe = 1;
 #endif /*WIN32*/
 
-  read_password("Password: ", flag_pipe, &password, &passlen);
+  if (!flag_disable) {
+      read_password("Password: ", flag_pipe, &password, &passlen);
 
-  if (! flag_pipe) {
-    read_password("Again (for verification): ", flag_pipe, &verify,
+      if (! flag_pipe) {
+	  read_password("Again (for verification): ", flag_pipe, &verify,
 		  &verifylen);
-    if (passlen != verifylen
-	|| memcmp(password, verify, verifylen)) {
-      fprintf(stderr, "%s: passwords don't match; aborting\n", progname);
-      exit(-SASL_BADPARAM);
-    }
+	  if (passlen != verifylen
+	      || memcmp(password, verify, verifylen)) {
+	      fprintf(stderr, "%s: passwords don't match; aborting\n", 
+		      progname);
+	      exit(-SASL_BADPARAM);
+	  }
+      }
   }
 
-  printf("setting pass\n");
   result = sasl_setpass(conn,
 			userid,
 			password,
@@ -252,7 +255,6 @@ main(int argc, char *argv[])
 			(flag_create ? SASL_SET_CREATE : 0)
 			| (flag_disable ? SASL_SET_DISABLE : 0),
 			&errstr);
-  printf("set pass\n");
 
   if (result != SASL_OK)
     exit_sasl(result, errstr);
